import json
import yaml
from datetime import datetime

from django.shortcuts import render
from django.db.models import Q
from django.http import JsonResponse, HttpResponse
from django.views.decorators.csrf import csrf_exempt

from backoffice.models import Authority, ReferenceType, Reference
from backoffice.serializers import ReferenceSerializer, LightReferenceSerializer


def index(request):
    authorities = Authority.objects.all()
    types = ReferenceType.objects.all()

    references = Reference.objects.all()

    context = {
        'authorities': authorities,
        'ref_types': types,
        'references': references
    }

    return render(request, 'webapp/index.html', context)

@csrf_exempt
def search(request):

    search_param = json.loads(request.POST['payload'])

    query_set = generate_search_querybuilder(search_param)

    serializer = LightReferenceSerializer(query_set, many=True)

    return JsonResponse(serializer.data, safe=False)

@csrf_exempt
def find(request, id):

    ref = Reference.objects.get(pk=id)
    serializer = ReferenceSerializer(ref)

    return JsonResponse(serializer.data, safe=False)


@csrf_exempt
def export(request):

    search_param = json.loads(request.POST['payload'])

    query_set = generate_search_querybuilder(search_param)

    formated_data = list()

    for ref in query_set:
        formated_data.append(ref.to_array())

    yml_data = yaml.dump({'references': formated_data}, explicit_start=True, encoding='utf8')

    response = HttpResponse(yml_data, content_type='text/yaml')
    response['Content-Disposition'] = 'filename=export.yml'
    return response

def export_global(request):
    refs = Reference.objects.all().filter(disabled=False).order_by('-update_date')

    formated_data = list()

    for ref in refs:
        formated_data.append(ref.to_array())

    last_update_date = refs[0].update_date

    yml_data = yaml.dump({'references': formated_data}, explicit_start=True, encoding='utf8')
    response = HttpResponse(yml_data, content_type='text/yaml')
    response['Last-Modified'] = last_update_date.strftime("%a, %d %b %Y %X GMT")
    response['Content-Disposition'] = 'filename=references.yml'
    return response

def empty(dic, key):
    return key not in dic or not dic[key]

def generate_search_querybuilder(search_param):
    query_set = Reference.objects.all().filter(disabled=False)

    if not empty(search_param, 'title'):
        query_set = query_set.filter(title_short__icontains=search_param['title'])

    if not empty(search_param, 'numero'):
        number = search_param['numero']
        query_set = query_set.filter(Q(identifier__icontains=number) | Q(number__icontains=number) | Q(ecli__icontains=number))

    if not empty(search_param, 'dateAfter'):
        try:
            date_after = datetime.strptime(search_param['dateAfter'], '%d/%m/%Y')
        except ValueError as e:
            pass
        else:
            query_set = query_set.filter(issued_date__gte=date_after)


    if not empty(search_param, 'dateBefore'):
        try:
            date_before = datetime.strptime(search_param['dateBefore'], '%d/%m/%Y')
        except ValueError as e:
            pass
        else:
            query_set = query_set.filter(issued_date__lte=date_before)




    query_set = query_set.filter(authority__slug__in=search_param['authorities'])

    query_set = query_set.filter(reference_type__slug__in=search_param['types'])

    return query_set



